﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Engineer
{
    class Rendevous
    {
        enum TargetType { None, Vessel, Debris, Celestial };

        FlightEngineer fe;
        Vessel targetVes;
        CelestialBody targetBody;
        Vector2 scrollPosition = Vector2.zero;
        TargetType typeOfTarget = TargetType.None;
        bool hasTarget = false;

        public Rendevous(FlightEngineer flightEngineer)
        {
            fe = flightEngineer;
        }

        public void Draw()
        {
            if (!hasTarget)
            {
                switch (typeOfTarget)
                {
                    case TargetType.Vessel:
                        DrawTargets();
                        break;
                    case TargetType.Celestial:
                        DrawTargets();
                        break;
                    case TargetType.Debris:
                        DrawTargets();
                        break;
                    default:
                        DrawTargetTypes();
                        break;
                }
            }
            else
            {
                DrawDetails();
            }
        }

        private void DrawTargetTypes()
        {
            GUILayout.Label("Rendezvous Display", fe.titleStyle);
            GUILayout.BeginHorizontal(fe.areaStyle);
            GUILayout.BeginVertical();
            if (GUILayout.Button("Vessels", fe.buttonStyle))
            {
                typeOfTarget = TargetType.Vessel;
                fe.winPos.height = 0;
            }
            if (GUILayout.Button("Debris", fe.buttonStyle))
            {
                typeOfTarget = TargetType.Debris;
                fe.winPos.height = 0;
            }
            if (GUILayout.Button("Celestial Bodies", fe.buttonStyle))
            {
                typeOfTarget = TargetType.Celestial;
                fe.winPos.height = 0;
            }
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();
        }

        private void DrawTargets()
        {
            if (typeOfTarget == TargetType.Celestial)
            {
                GUILayout.Label("Cellestial Bodies", fe.titleStyle);
                GUILayout.BeginHorizontal(fe.areaStyle);
                GUILayout.BeginVertical();

                List<CelestialBody> planets = new List<CelestialBody>();

                foreach (CelestialBody body in FlightGlobals.Bodies)
                {
                    if (body.referenceBody.name != "Sun" && body.referenceBody != fe.activeVessel.orbit.referenceBody) continue;
                    if (body == fe.activeVessel.orbit.referenceBody) continue;
                    if (body.name == "Sun") continue;

                    if (body.referenceBody != fe.activeVessel.orbit.referenceBody)
                    {
                        planets.Add(body);
                    }
                    else
                    {
                        if (GUILayout.Button(body.name, fe.buttonStyle))
                        {
                            targetBody = body;
                            hasTarget = true;
                            fe.winPos.height = 0;
                            return;
                        }
                    }
                }

                GUILayout.Space(5);

                foreach (CelestialBody body in planets)
                {
                    if (GUILayout.Button(body.name, fe.buttonStyle))
                    {
                        targetBody = body;
                        hasTarget = true;
                        fe.winPos.height = 0;
                        return;
                    }
                }
            }
            else if (typeOfTarget != TargetType.None)
            {
                if (typeOfTarget == TargetType.Debris)
                    GUILayout.Label("Debris", fe.titleStyle);

                if (typeOfTarget == TargetType.Vessel)
                    GUILayout.Label("Vessels", fe.titleStyle);

                GUILayout.BeginHorizontal(fe.areaStyle);
                GUILayout.BeginVertical();
                foreach (Vessel v in FlightGlobals.Vessels)
                {
                    if (v == fe.activeVessel) continue;
                    if (v.LandedOrSplashed) continue;

                    if (typeOfTarget == TargetType.Debris)
                        if (v.isCommandable) continue;

                    if (typeOfTarget == TargetType.Vessel)
                        if (!v.isCommandable) continue;

                    if (v.mainBody == fe.activeVessel.mainBody)
                    {
                        if (GUILayout.Button(v.vesselName, fe.buttonStyle))
                        {
                            targetVes = v;
                            hasTarget = true;
                            fe.winPos.height = 0;
                            return;
                        }
                    }
                }

            }

            GUILayout.EndVertical();
            GUILayout.EndHorizontal();

            if (GUILayout.Button("Back to Target Type", fe.buttonStyle))
            {
                targetVes = null;
                targetBody = null;
                hasTarget = false;
                typeOfTarget = TargetType.None;
                fe.winPos.height = 0;
            }
        }

        private void DrawDetails()
        {
            Vector3d active = Vector3d.zero;
            Vector3d target = Vector3d.zero;
            string name = null;
            double phaseAngle = 0.0f;
            double distance = 0.0f;
            double relInclination = 0.0f;
            double altitude = 0.0f;
            double relVelocity = 0.0f;
            double gravity = 0.0f;

            if (typeOfTarget == TargetType.Vessel || typeOfTarget == TargetType.Debris)
            {
                name = targetVes.vesselName;
                phaseAngle = Vector3d.Angle(fe.activeVessel.orbit.referenceBody.position - fe.activeVessel.CoM, targetVes.orbit.referenceBody.position - targetVes.CoM);
                relInclination = targetVes.orbit.inclination - fe.activeVessel.orbit.inclination;
                altitude = targetVes.orbit.altitude;
                relVelocity = targetVes.orbit.vel.magnitude - fe.activeVessel.orbit.vel.magnitude;
                distance = Vector3d.Distance(fe.activeVessel.CoM, targetVes.CoM);
            }
            else if (typeOfTarget == TargetType.Celestial)
            {
                name = targetBody.name;
                Vector3d vectorVessel = Vector3d.zero;
                Vector3d vectorTarget = Vector3d.zero;

                if ((fe.activeVessel.mainBody.name == "Sun") || fe.activeVessel.mainBody == targetBody.referenceBody)
                {
                    vectorVessel = fe.activeVessel.orbit.getRelativePositionAtUT(Planetarium.GetUniversalTime());
                    vectorTarget = targetBody.orbit.getRelativePositionAtUT(Planetarium.GetUniversalTime());
                }
                else
                {
                    vectorVessel = fe.activeVessel.mainBody.orbit.getRelativePositionAtUT(Planetarium.GetUniversalTime());
                    vectorTarget = targetBody.orbit.getRelativePositionAtUT(Planetarium.GetUniversalTime());
                }

                phaseAngle = Vector3d.Angle(vectorVessel, vectorTarget);
                relInclination = targetBody.orbit.inclination - fe.activeVessel.orbit.inclination;
                altitude = targetBody.orbit.altitude;
                distance = Vector3d.Distance(fe.activeVessel.CoM, targetBody.position);
                gravity = targetBody.gMagnitudeAtCenter;
            }

            GUILayout.Label(name, fe.titleStyle);

            GUILayout.BeginHorizontal(fe.areaStyle);
            GUILayout.BeginVertical();
            GUILayout.Label("Phase Angle", fe.headStyle);
            GUILayout.Label("Relative Inclination", fe.headStyle);
            GUILayout.Label("Altitude", fe.headStyle);
            GUILayout.Label("Distance", fe.headStyle);
            if (typeOfTarget != TargetType.Celestial)
                GUILayout.Label("Relative Velocity", fe.headStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label(phaseAngle.ToString("0.000") + "°", fe.dataStyle);
            GUILayout.Label(relInclination.ToString("0.000") + "°", fe.dataStyle);
            GUILayout.Label(fe.FormatDistance(altitude), fe.dataStyle);
            GUILayout.Label(fe.FormatDistance(distance), fe.dataStyle);
            if (typeOfTarget != TargetType.Celestial)
                GUILayout.Label(fe.FormatSpeed(relVelocity), fe.dataStyle);
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();

            if (GUILayout.Button("Back to Targets", fe.buttonStyle))
            {
                targetVes = null;
                targetBody = null;
                hasTarget = false;
                fe.winPos.height = 0;
            }
        }

        public CelestialBody GetBodyByName(string name)
        {
            foreach (CelestialBody body in FlightGlobals.Bodies)
            {
                if (body.name == name)
                {
                    return body;
                }
            }

            return null;
        }
    }
}
